package com.jkcredit.asychronous.job;

import com.alibaba.fastjson.JSONArray;
import com.jkcredit.asychronous.entity.Man;
import com.jkcredit.asychronous.service.ManService;
import com.jkcredit.asychronous.util.ExcelUtil;
import com.jkcredit.asychronous.util.SFTPUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Descirption
 * @Author Zhaoning Sun
 * @Date Created in 2018/6/22 下午7:34
 */
@Slf4j
@Component
public class ManJob {

    @Value("${manftp.upload.local}")
    private String localDir;
    @Value("${ftp.host}")
    private String host;
    @Value("${ftp.port}")
    private Integer port;
    @Value("${ftp.username}")
    private String userName;
    @Value("${ftp.passphrase}")
    private String passphrase;
    @Value("${ftp.dir.man}")
    private String dir;
    @Value("${ftp.keyFilePath}")
    private String keyFilePath;
    @Value("${ftp.dir.manDownload}")
    private String manDownloadPath;
    @Value("${manftp.download.info}")
    private String fileDir;

    @Autowired
    private ManService manService;

    @Scheduled(cron = "0/10 * * * * ?")
    public void man() {
        Long beginTime= System.currentTimeMillis() - 10000;
        Long endTime = System.currentTimeMillis();
        List<Man> list = manService.selectByTime(beginTime,endTime);
        if (list.size() > 0) {
            JSONArray ja = new JSONArray();
            for(Man man : list){
                ja.add(man);
            }
            Map<String,String> headMap = new LinkedHashMap<>();
            headMap.put("id", "序列号");
            headMap.put("name","姓名");
            headMap.put("idCard","身份证号");
            String fileName = localDir + File.separator + endTime.toString() + ".xlsx";
            try {
                OutputStream outXlsx = new FileOutputStream(fileName);
                ExcelUtil.exportExcelX(headMap,ja,null,0,outXlsx);
                outXlsx.close();
            } catch (FileNotFoundException e) {
                e.printStackTrace();
                log.info("FileNotFoundException找不到路径");
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                log.info("文件上传 路径：{}", File.separator + dir + File.separator +  endTime.toString() + ".xlsx", fileName);
                SFTPUtil.upload(host, port, userName, passphrase, File.separator + dir + File.separator +  endTime.toString() + ".xlsx", fileName, keyFilePath);
                // todo 上传完删除本地文件
                File localFile = new File(fileName);
                if (localFile.exists()) {
                    localFile.delete();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    @Scheduled(cron = "0 19 16 * * ?", zone = "Asia/Shanghai")
    public void test(){

        SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");//设置日期格式

        List<Man> list = new ArrayList<>();
        Man mantest = new Man();
        mantest.setIdCard("420581198402220816");
        mantest.setName("徐建华");
        list.add(mantest);
        JSONArray ja = new JSONArray();
        for(Man man : list){
            ja.add(man);
        }
        Map<String,String> headMap = new LinkedHashMap<>();
        headMap.put("name","姓名");
        headMap.put("idCard","身份证号");
        String fileName = localDir + File.separator + df.format(new Date()) + ".xlsx";

        try {
            OutputStream outXlsx = new FileOutputStream(fileName);
            ExcelUtil.exportExcelX(headMap,ja,null,0,outXlsx);
            outXlsx.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
//            String fileName = ExcelUtil.export(list, Man.class, localDir,endTime.toString());
//            log.info("ManExcel：{}", fileName);
//        SFTPUtil.upload(host, port, userName, passphrase, dir, fileName);
    }

    @Scheduled(cron = "0 0/1 * * * ?", zone = "Asia/Shanghai")
    private void downloadFile() {
        try {
            log.info("下载文件 localDir == " + fileDir);
            log.info("下载文件 manDownloadPath == " + manDownloadPath);
            SFTPUtil.download(userName, port, host, passphrase, keyFilePath, fileDir, manDownloadPath);
        } catch (Exception e) {
            log.error("下载文件失败，失败原因：{}", e.getMessage());
        }
    }

}
