package com.jkcredit.asychronous.process;

import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import com.jkcredit.asychronous.common.ResponseEntity;
import com.jkcredit.asychronous.entity.CarResult;
import com.jkcredit.asychronous.service.CarResultService;
import lombok.extern.slf4j.Slf4j;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.component.file.GenericFileMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.io.*;
import java.util.List;

/**
 * @Descirption
 * @Author Zhaoning Sun
 * @Date Created in 2018/6/22 下午1:20
 */
@Component
@Slf4j
public class CarLocationFileProcessor implements Processor {

    @Value("${carftp.download.info}")
    private String fileDir;
//    @Value("${callback.url}")
    private String url;

    @Autowired
    private CarResultService carResultService;
    @Autowired
    private RestTemplate restTemplate;

    @Override
    public void process(Exchange exchange) {
        exchange.setProperty(Exchange.CHARSET_NAME, "UTF-8");
        GenericFileMessage<RandomAccessFile> inFileMessage = (GenericFileMessage<RandomAccessFile>) exchange.getIn();
        String fileName = inFileMessage.getGenericFile().getFileName();//文件名
        String splitTag = File.separator;//系统文件分隔符
        log.info(fileDir + splitTag + fileName);//文件的绝对路径

        List<CarResult> carlList = readExcel(fileDir + splitTag + fileName);
        if (carlList.size() > 0) {
            log.info("读取成功");
            carResultService.insert(carlList);
            log.info("入库成功");
            MultiValueMap<String, Object> paramMap = new LinkedMultiValueMap<>();
            paramMap.add("data", new ResponseEntity(carlList, "car"));
            HttpHeaders headers = new HttpHeaders();
            HttpEntity<MultiValueMap<String, Object>> httpEntity = new HttpEntity<>(paramMap, headers);
            restTemplate.exchange(url, HttpMethod.POST, httpEntity, String.class);
        } else {
            log.error("error", new ResponseEntity(400, "读取文件为空", ""));
        }

    }

    // 去读Excel的方法readExcel，该方法的入口参数为一个File对象
    public List<CarResult> readExcel(String file) {
        ImportParams params = new ImportParams();
        params.setHeadRows(1);
        List<CarResult> list = ExcelImportUtil.importExcel(
                new File(file),
                CarResult.class, params);
        return list;
    }

}
