package com.jkcredit.asychronous.util;

import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StringUtils;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @description:
 * @author: xusonglin
 * @create: 2020/2/3 17:19
 * @version: V1.0
 **/
@Slf4j
public class SFTPUtil {
    static JSch jsch = null;
    static Session session = null;
    static ChannelSftp channel = null;
    /**
     * 密钥连接到指定的IP
     *
     * @param keyFilePath 密钥路径
     * @param passphrase  密钥的密码
     */
    public static void connect(String userName, String host, int port, String passphrase, String keyFilePath) throws Exception {
        try {
            jsch = new JSch();
            if (keyFilePath != null) {
                if (passphrase != null) {
                    jsch.addIdentity(keyFilePath, passphrase);// 设置私钥
                } else {
                    jsch.addIdentity(keyFilePath);// 设置私钥
                }
                log.info("连接sftp，私钥文件路径：" + keyFilePath);
            }
            log.info("SFTP Host: " + host + "; UserName:" + userName);
            session = jsch.getSession(userName, host, port);
            log.debug("Session 已建立.");
            Properties sshConfig = new Properties();
            sshConfig.put("StrictHostKeyChecking", "no");
            session.setConfig(sshConfig);
            session.setConfig("kex", "diffie-hellman-group1-sha1");
            session.connect();
            log.debug("Session 已连接.");
            channel = (ChannelSftp) session.openChannel("sftp");
            channel.connect(60000);
            log.info("连接到SFTP成功.Host: " + host);
        } catch (Exception e) {
            log.error("连接SFTP失败：", e);
        }
    }

    /**
     * 关闭连接
     */
    public static void close() {
        if (channel != null) {
            try {
                channel.disconnect();
            } catch (Exception e) {
                log.error("sftp close channel failed...", e);
            }
        }
        if (session != null) {
            try {
                session.disconnect();
            } catch (Exception e) {
                log.error("sftp close session failed...", e);
            }
        }
    }

    public static void upload(String host, Integer port, String userName, String passphrase, String dir,String fileName, String keyFilePath) throws Exception {
        //连接sftp
        connect(userName, host, port, passphrase, keyFilePath);
        if (channel == null) {
            log.info("SFTP服务器未连接");
            return;
        }
        try {
//            fileName = "C:\\Users\\Administrator\\Desktop\\ftp\\upload\\1580697240002.xlsx";
//            channel.put(new FileInputStream(fileName), "/download/1580697240002.xlsx", ChannelSftp.OVERWRITE);
            log.info("dis == " + dir);
            channel.put(new FileInputStream(fileName), dir, ChannelSftp.OVERWRITE);
            log.info("上传到sftp成功");
        } catch (Exception e) {
            log.info("上传文件：" + fileName + "失败");
            throw e;
        } finally {
            close();
        }

        // todo 秘钥文件路径有问题 需要测试上传  下载需要在util中写个方法，替换ftp文件夹下的route
    }

    /**
     * 下载文件
     * 备份本地文件，再下载服务器端最新文件
     * @param localPath 本地存放路径
     * @param remotePath 服务器端存放路径
     */
    public static void download(String username, int port, String host, String passphrase, String keyFilePath,String localPath, String remotePath) throws Exception {
        // src linux服务器文件地址，dst 本地存放地址
        connect(username, host, port, passphrase, keyFilePath);
        if (channel == null) {
            log.info("SFTP服务器未连接");
            return;
        }
        FileOutputStream output;

        // 获取未读取文件
        try {
            List<String> files = listFiles(remotePath);
            for (String fileName : files) {
                output = new FileOutputStream(localPath + "/" + fileName);
                log.info("下载文件：" + fileName + "开始");
                channel.get(remotePath + "/" +fileName, output);
                log.info("下载文件：" + fileName + "成功");
                delete(remotePath, fileName);
                output.close();
            }
        } catch (Exception e) {
            log.error("下载文件失败,失败原因：{}", e.getMessage());
        } finally {
            close();
        }
    }

    /**
     * 删除文件
     *
     * @param directory
     *            要删除文件所在目录
     * @param deleteFile
     *            要删除的文件
     * @throws JSchException
     */
    private static void delete(String directory, String deleteFile) throws Exception {
        if (channel == null) {
            log.info("SFTP服务器未连接");
            return;
        }
        channel.cd(directory);
        channel.rm(deleteFile);
        log.info("删除成功");
    }

    /**
     * 列出目录下的文件
     *
     * @param directory
     *            要列出的目录
     * @param sftp
     * @return
     * @throws JSchException
     */
    @SuppressWarnings("rawtypes")
    private static List<String> listFiles(String directory) throws Exception {
        if (channel == null) {
            log.info("SFTP服务器未连接");
            return null;
        }
        Vector vector = channel.ls(directory);

        List<String> fileNameList = new ArrayList<>();
        for (Object object : vector) {
            String fileName = ((ChannelSftp.LsEntry) object).getFilename();
            String[] array = StringUtils.split(fileName,".");
            if (array.length == 2 && (array[1].equals("xlsx") || array[1].equals("xls"))) {
                fileNameList.add(fileName);
            }
        }
        return fileNameList;
    }
}
